/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal.navigator;

import org.conqat.ide.commons.ui.logging.LoggingUtils;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.AdapterImpl;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.DecoratingLabelProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IDecoratorManager;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.progress.UIJob;
import org.fortiss.tooling.kernel.ToolingKernelActivator;
import org.unicase.ecp.model.ECPWorkspaceManager;
import org.unicase.ecp.model.NoWorkspaceException;
import org.unicase.ecp.model.workSpaceModel.ECPProject;
import org.unicase.ecp.model.workSpaceModel.ECPProjectListener;
import org.unicase.ecp.model.workSpaceModel.ECPWorkspace;
import org.unicase.ecp.model.workSpaceModel.WorkSpaceModelPackage;

/**
 * {@link ViewPart} of the model navigator provided by the tooling kernel.
 * 
 * @author hoelzlf
 * @author $Author$
 * @version $Rev$
 * @levd.rating RED Rev:
 */
public final class NavigatorViewPart extends ViewPart implements
		ISelectionListener, ISelectionChangedListener {

	/** Stores the TreeViewer. */
	private TreeViewer viewer;

	/** Stores the EmfStore workspace. */
	private ECPWorkspace workspace;

	/** Stores the workspace listener. */
	private AdapterImpl workspaceListener;

	/** Stores the project listener. */
	private ECPProjectListener projectListener;

	/** Stores the menu manager. */
	private MenuManager menuManager;

	/** Constructor. */
	public NavigatorViewPart() {
		try {
			workspace = ECPWorkspaceManager.getInstance().getWorkSpace();
		} catch (NoWorkspaceException e) {
			LoggingUtils.log(ToolingKernelActivator.getDefault(),
					e.getMessage(), IStatus.ERROR, e);
			return;
		}

		createProjectListener();
		for (ECPProject project : workspace.getProjects()) {
			project.addECPProjectListener(projectListener);
		}

		createWorkspaceListener();
		workspace.eAdapters().add(workspaceListener);
	}

	/**
	 * Creates the workspace listener, which registers the project listener on
	 * added projects. It also unregisters the project listener from removed
	 * projects.
	 */
	private void createWorkspaceListener() {
		workspaceListener = new AdapterImpl() {

			@Override
			public void notifyChanged(Notification msg) {
				if ((msg.getFeatureID(ECPWorkspace.class)) == WorkSpaceModelPackage.ECP_WORKSPACE__PROJECTS) {
					if (msg.getEventType() == Notification.ADD
							&& WorkSpaceModelPackage.eINSTANCE.getECPProject()
									.isInstance(msg.getNewValue())) {
						ECPProject projectSpace = (ECPProject) msg
								.getNewValue();
						projectSpace.addECPProjectListener(projectListener);
					} else if (msg.getEventType() == Notification.REMOVE
							&& WorkSpaceModelPackage.eINSTANCE.getECPProject()
									.isInstance(msg.getOldValue())) {
						ECPProject projectSpace = (ECPProject) msg
								.getOldValue();
						projectSpace.removeECPProjectListener(projectListener);
					}
				}
				super.notifyChanged(msg);
			}
		};
	}

	/**
	 * Creates the project listener, which refreshes the viewer upon any
	 * changes.
	 */
	private void createProjectListener() {
		projectListener = new ECPProjectListener() {
			@Override
			public void projectDeleted() {
				// ignore
			}

			@Override
			public void projectChanged() {
				new UIJob("Update Model Navigator") {

					@Override
					public IStatus runInUIThread(IProgressMonitor monitor) {
						viewer.refresh();
						return Status.OK_STATUS;
					}
				}.schedule();
			}

			@Override
			public void modelelementDeleted(EObject eobject) {
				// ignore
			}
		};
	}

	/** {@inheritDoc} */
	@Override
	public void createPartControl(Composite parent) {
		viewer = new TreeViewer(parent, SWT.MULTI);

		if (workspace != null) {
			IDecoratorManager decoratorManager = PlatformUI.getWorkbench()
					.getDecoratorManager();
			viewer.setLabelProvider(new DecoratingLabelProvider(
					new NavigatorTreeLabelProvider(), decoratorManager
							.getLabelDecorator()));
			viewer.setContentProvider(new NavigatorTreeContentProvider());
			viewer.setInput(workspace);
		}

		createContextMenu();

		PlatformUI.getWorkbench().getActiveWorkbenchWindow()
				.getSelectionService().addSelectionListener(this);
		getSite().setSelectionProvider(viewer);

		// FIXME: whats that good for?
		// if (viewer.getTree().getItems().length > 0) {
		// setActiveECPProject(viewer.getTree().getItem(0).getData());
		// viewer.getTree().select(viewer.getTree().getItem(0));
		// }
	}

	/** Creates the context menu. */
	private void createContextMenu() {
		menuManager = new MenuManager();

		MenuManager newMenuManager = new MenuManager("New ...",
				ToolingKernelActivator.getImageDescriptor("icons/add.png"),
				NewMenu.MENU_ID);
		newMenuManager.add(new NewMenu());
		menuManager.add(newMenuManager);

		Separator repositorySection = new Separator("repository");
		repositorySection.setVisible(true);
		menuManager.add(repositorySection);

		Separator additions = new Separator(
				IWorkbenchActionConstants.MB_ADDITIONS);
		additions.setVisible(true);
		menuManager.add(additions);

		getSite().registerContextMenu(menuManager, viewer);

		Control control = viewer.getControl();
		Menu menu = menuManager.createContextMenu(control);
		control.setMenu(menu);
	}

	/** Creates the actions. */

	/** {@inheritDoc} */
	@Override
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	/** {@inheritDoc} */
	@Override
	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		// TODO (FH): implement link with editor feature here
	}

	/** {@inheritDoc} */
	@Override
	public void selectionChanged(SelectionChangedEvent event) {
		if (event.getSelection() instanceof IStructuredSelection) {
			IStructuredSelection selection = (IStructuredSelection) event
					.getSelection();
			Object obj = selection.getFirstElement();
			// FIXME: Whats that good for?
			// setActiveECPProject(obj);
		}
	}

	/** Sets the active ECPProject element. */
	private void setActiveECPProject(Object obj) {
		if (obj instanceof EObject) {
			try {
				ECPWorkspaceManager.getInstance().getWorkSpace()
						.setActiveModelelement((EObject) obj);
			} catch (NoWorkspaceException e) {
				LoggingUtils.error(ToolingKernelActivator.getDefault(),
						e.getMessage(), e);
			}
		}

	}

	/** {@inheritDoc} */
	@Override
	public void dispose() {
		PlatformUI.getWorkbench().getActiveWorkbenchWindow()
				.getSelectionService().removeSelectionListener(this);
		getSite().setSelectionProvider(null);

		workspace.eAdapters().remove(workspaceListener);
		for (ECPProject project : workspace.getProjects()) {
			project.removeECPProjectListener(projectListener);
		}

		super.dispose();
	}
}
